==> reactor-extra-3.4.6-sources.jar: reactor/math/MathSubscriber.java <==
/*
 * Copyright (c) 2017-2021 VMware Inc. or its affiliates, All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package reactor.math;

import org.reactivestreams.Subscription;
import reactor.core.CoreSubscriber;
import reactor.core.publisher.Operators;

abstract class MathSubscriber<T, R> extends Operators.MonoSubscriber<T, R> {

	Subscription s;

	boolean done;

	MathSubscriber(CoreSubscriber<? super R> actual) {
		super(actual);
	}

	@Override
	public void onSubscribe(Subscription s) {
		if (Operators.validate(this.s, s)) {
			this.s = s;
			actual.onSubscribe(this);
			s.request(Long.MAX_VALUE);
		}
	}

	@Override
	public void onNext(T t) {
		if (done) {
			Operators.onNextDropped(t, actual.currentContext());
			return;
		}

		try {
			updateResult(t);
		}
		catch (Throwable ex) {
			reset();
			done = true;
			actual.onError(Operators.onOperatorError(s, ex, t, actual.currentContext()));
			return;
		}
	}

	@Override
	public void onError(Throwable t) {
		if (done) {
			Operators.onErrorDropped(t, actual.currentContext());
			return;
		}
		done = true;
		reset();
		actual.onError(t);
	}

	@Override
	public void onComplete() {
		if (done) {
			return;
		}
		done = true;
		R r = result();
		if (r != null) {
			complete(r);
		}
		else {
			actual.onComplete();
		}
	}

	@Override
	public void cancel() {
		super.cancel();
		s.cancel();
	}

	protected abstract void reset();

	protected abstract R result();

	protected abstract void updateResult(T newValue);
}

==> reactor-extra-3.6.0-sources.jar: reactor/math/MathSubscriber.java <==
/*
 * Copyright (c) 2017-2025 VMware Inc. or its affiliates, All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package reactor.math;

import org.jspecify.annotations.Nullable;
import org.reactivestreams.Subscription;
import reactor.core.CoreSubscriber;
import reactor.core.publisher.Operators;

abstract class MathSubscriber<T, R> extends Operators.MonoSubscriber<T, R> {

	@SuppressWarnings("DataFlowIssue") // s is initialized in onSubscribe
	Subscription s;

	boolean done;

	MathSubscriber(CoreSubscriber<? super R> actual) {
		super(actual);
	}

	@Override
	public void onSubscribe(Subscription s) {
		if (Operators.validate(this.s, s)) {
			this.s = s;
			actual.onSubscribe(this);
			s.request(Long.MAX_VALUE);
		}
	}

	@Override
	public void onNext(T t) {
		if (done) {
			Operators.onNextDropped(t, actual.currentContext());
			return;
		}

		try {
			updateResult(t);
		}
		catch (Throwable ex) {
			reset();
			done = true;
			actual.onError(Operators.onOperatorError(s, ex, t, actual.currentContext()));
			return;
		}
	}

	@Override
	public void onError(Throwable t) {
		if (done) {
			Operators.onErrorDropped(t, actual.currentContext());
			return;
		}
		done = true;
		reset();
		actual.onError(t);
	}

	@Override
	public void onComplete() {
		if (done) {
			return;
		}
		done = true;
		R r = result();
		if (r != null) {
			complete(r);
		}
		else {
			actual.onComplete();
		}
	}

	@Override
	public void cancel() {
		super.cancel();
		s.cancel();
	}

	protected abstract void reset();

	protected abstract @Nullable R result();

	protected abstract void updateResult(T newValue);
}
